@@ -84,32 +84,32 @@ AbstractMetaType *AbstractMetaType::copy() const
 
 QString AbstractMetaType::cppSignature() const
 {
-    QString s;
+    if (m_cachedCppSignature.isEmpty()) {
+        if (isConstant())
+            m_cachedCppSignature += "const ";
 
-    if (isConstant())
-        s += "const ";
+        m_cachedCppSignature += typeEntry()->qualifiedCppName();
 
-    s += typeEntry()->qualifiedCppName();
-
-    if (hasInstantiationInCpp()) {
-        AbstractMetaTypeList types = instantiations();
-        s += "<";
-        for (int i = 0; i < types.count(); ++i) {
-            if (i > 0)
-                s += ", ";
-            s += types[i]->cppSignature();
+        if (hasInstantiationInCpp()) {
+            AbstractMetaTypeList types = instantiations();
+            m_cachedCppSignature += "<";
+            for (int i = 0; i < types.count(); ++i) {
+                if (i > 0)
+                    m_cachedCppSignature += ", ";
+                m_cachedCppSignature += types[i]->cppSignature();
+            }
+            m_cachedCppSignature += " >";
         }
-        s += " >";
-    }
 
-    if (actualIndirections()) {
-        s += ' ';
-        if (indirections())
-            s += QString(indirections(), '*');
-        if (isReference())
-            s += '&';
+        if (actualIndirections()) {
+            m_cachedCppSignature += ' ';
+            if (indirections())
+                m_cachedCppSignature += QString(indirections(), '*');
+            if (isReference())
+                m_cachedCppSignature += '&';
+        }
     }
-    return s;
+    return m_cachedCppSignature;
 }
 
 
@@ -326,26 +326,27 @@ QStringList AbstractMetaFunction::introspectionCompatibleSignatures(const QStrin
 
 QString AbstractMetaFunction::signature() const
 {
-    QString s(m_originalName);
+    if (m_cachedSignature.isEmpty()) {
+        m_cachedSignature = m_originalName;
 
-    s += "(";
+        m_cachedSignature += '(';
 
-    for (int i = 0; i < m_arguments.count(); ++i) {
-        if (i > 0)
-            s += ", ";
-        AbstractMetaArgument *a = m_arguments.at(i);
-        s += a->type()->cppSignature();
+        for (int i = 0; i < m_arguments.count(); ++i) {
+            if (i > 0)
+                m_cachedSignature += ", ";
+            AbstractMetaArgument *a = m_arguments.at(i);
+            m_cachedSignature += a->type()->cppSignature();
 
-        // We need to have the argument names in the qdoc files
-        s += " ";
-        s += a->name();
+            // We need to have the argument names in the qdoc files
+            m_cachedSignature += ' ';
+            m_cachedSignature += a->name();
+        }
+        m_cachedSignature += ")";
+
+        if (isConstant())
+            m_cachedSignature += " const";
     }
-    s += ")";
-
-    if (isConstant())
-        s += " const";
-
-    return s;
+    return m_cachedSignature;
 }
 
 int AbstractMetaFunction::actualMinimumArgumentCount() const
@@ -708,6 +709,11 @@ QString AbstractMetaFunction::argumentName(int index, bool create, const Abstrac
     return m_arguments[--index]->name();
 }
 
+bool AbstractMetaFunction::isCallOperator() const
+{
+    return m_name == "operator()";
+}
+
 bool AbstractMetaFunction::hasInjectedCode() const
 {
     foreach (const FunctionModification mod, modifications(ownerClass())) {
@@ -750,7 +756,7 @@ bool AbstractMetaFunction::hasSignatureModifications() const
 
 bool AbstractMetaFunction::isConversionOperator(QString funcName)
 {
-    QRegExp opRegEx("^operator(?:\\s+(?:const|volatile))?\\s+(\\w+\\s*)&?$");
+    static QRegExp opRegEx("^operator(?:\\s+(?:const|volatile))?\\s+(\\w+\\s*)&?$");
     return opRegEx.indexIn(funcName) > -1;
 }
 
@@ -759,7 +765,7 @@ bool AbstractMetaFunction::isOperatorOverload(QString funcName)
     if (isConversionOperator(funcName))
         return true;
 
-    QRegExp opRegEx("^operator([+\\-\\*/%=&\\|\\^\\<>!][=]?"
+    static QRegExp opRegEx("^operator([+\\-\\*/%=&\\|\\^\\<>!][=]?"
                     "|\\+\\+|\\-\\-|&&|\\|\\||<<[=]?|>>[=]?|~"
                     "|\\[\\]|\\s+delete\\[?\\]?"
                     "|\\(\\)"
@@ -778,6 +784,11 @@ bool AbstractMetaFunction::isArithmeticOperator() const
         return false;
 
     QString name = originalName();
+
+    // It's a dereference operator!
+    if (name == "operator*" && m_arguments.isEmpty())
+        return false;
+
     return name == "operator+" || name == "operator+="
             || name == "operator-" || name == "operator-="
             || name == "operator*" || name == "operator*="
@@ -854,7 +865,7 @@ bool AbstractMetaFunction::isOtherOperator() const
 
 int AbstractMetaFunction::arityOfOperator() const
 {
-    if (!isOperatorOverload())
+    if (!isOperatorOverload() || isCallOperator())
         return -1;
 
     int arity = m_arguments.size();
@@ -1335,6 +1346,7 @@ bool AbstractMetaClass::hasDefaultToStringFunction() const
 
 void AbstractMetaClass::addFunction(AbstractMetaFunction *function)
 {
+    Q_ASSERT(!function->signature().startsWith("("));
     function->setOwnerClass(this);
 
     if (!function->isDestructor())
@@ -1478,6 +1490,23 @@ AbstractMetaField *AbstractMetaField::copy() const
     return returned;
 }
 
+/*******************************************************************************
+ * Indicates that this field has a modification that removes it
+ */
+bool AbstractMetaField::isModifiedRemoved(int types) const
+{
+    FieldModificationList mods = modifications();
+    foreach (FieldModification mod, mods) {
+        if (!mod.isRemoveModifier())
+            continue;
+
+        if ((mod.removal & types) == types)
+            return true;
+    }
+
+    return false;
+}
+
 static QString upCaseFirst(const QString &str)
 {
     Q_ASSERT(!str.isEmpty());
@@ -1596,6 +1625,7 @@ bool AbstractMetaClass::hasPrivateCopyConstructor() const
 void AbstractMetaClass::addDefaultConstructor()
 {
     AbstractMetaFunction *f = new AbstractMetaFunction;
+    f->setOriginalName(name());
     f->setName(name());
     f->setOwnerClass(this);
     f->setFunctionType(AbstractMetaFunction::ConstructorFunction);
@@ -1616,6 +1646,7 @@ void AbstractMetaClass::addDefaultConstructor()
 void AbstractMetaClass::addDefaultCopyConstructor(bool isPrivate)
 {
     AbstractMetaFunction* f = new AbstractMetaFunction;
+    f->setOriginalName(name());
     f->setName(name());
     f->setOwnerClass(this);
     f->setFunctionType(AbstractMetaFunction::ConstructorFunction);
@@ -1845,7 +1876,7 @@ AbstractMetaEnum *AbstractMetaClass::findEnum(const QString &enumName)
 AbstractMetaEnumValue *AbstractMetaClass::findEnumValue(const QString &enumValueName, AbstractMetaEnum *meta_enum)
 {
     foreach (AbstractMetaEnum *e, m_enums) {
-        if (e == meta_enum)
+        if (e != meta_enum)
             continue;
         foreach (AbstractMetaEnumValue *v, e->values()) {
             if (v->name() == enumValueName)
@@ -1960,6 +1991,10 @@ void AbstractMetaClass::fixFunctions()
             if (sf->isRemovedFromAllLanguages(sf->implementingClass()))
                 continue;
 
+            // skip functions added in base classes
+            if (sf->isUserAdded() && sf->declaringClass() != this)
+                continue;
+
             // we generally don't care about private functions, but we have to get the ones that are
             // virtual in case they override abstract functions.
             bool add = (sf->isNormal() || sf->isSignal() || sf->isEmptyFunction());
@@ -1968,6 +2003,7 @@ void AbstractMetaClass::fixFunctions()
                 if (f->isRemovedFromAllLanguages(f->implementingClass()))
                     continue;
 
+
                 uint cmp = f->compareTo(sf);
 
                 if (cmp & AbstractMetaFunction::EqualModifiedName) {
@@ -2216,15 +2252,22 @@ AbstractMetaEnumValue *AbstractMetaClassList::findEnumValue(const QString &name)
 {
     QStringList lst = name.split(QLatin1String("::"));
 
-    Q_ASSERT_X(lst.size() == 2, "AbstractMetaClassList::findEnumValue()", "Expected qualified enum");
+    if (lst.size() > 1) {
+        QString prefixName = lst.at(0);
+        QString enumName = lst.at(1);
 
+        AbstractMetaClass* cl = findClass(prefixName);
+        if (cl)
+            return cl->findEnumValue(enumName, 0);
+    }
 
-    QString prefixName = lst.at(0);
-    QString enumName = lst.at(1);
-
-    AbstractMetaClass *cl = findClass(prefixName);
-    if (cl)
-        return cl->findEnumValue(enumName, 0);
+    foreach(AbstractMetaClass* metaClass, *this) {
+        foreach(AbstractMetaEnum* metaEnum, metaClass->enums()) {
+            AbstractMetaEnumValue* enumValue = metaClass->findEnumValue(name, metaEnum);
+            if (enumValue)
+                return enumValue;
+        }
+    }
 
     ReportHandler::warning(QString("no matching enum '%1'").arg(name));
     return 0;
