@@ -28,24 +28,7 @@
 
 Documentation QtDocParser::retrieveModuleDocumentation()
 {
-    // TODO: This method of acquiring the module name supposes that the target language uses
-    // dots as module separators in package names. Improve this.
-    QString moduleName = QString(packageName()).remove(0, packageName().lastIndexOf('.') + 1);
-    QString sourceFile = documentationDataDirectory() + '/' + moduleName.toLower() + ".xml";
-
-    if (!QFile::exists(sourceFile)) {
-        ReportHandler::warning("Can't find qdoc3 file for module "
-                               + packageName() + ", tried: "
-                               + sourceFile);
-        return Documentation();
-    }
-
-    QXmlQuery xquery;
-    xquery.setFocus(QUrl(sourceFile));
-
-    // Module documentation
-    QString query = "/WebXML/document/page[@name=\"" + moduleName + "\"]/description";
-    return Documentation(getDocumentation(xquery, query, DocModificationList()));
+    return retrieveModuleDocumentation(packageName());
 }
 
 void QtDocParser::fillDocumentation(AbstractMetaClass* metaClass)
@@ -62,10 +45,6 @@ void QtDocParser::fillDocumentation(AbstractMetaClass* metaClass)
     }
 
     QString filename = metaClass->qualifiedCppName().toLower().replace("::", "-");
-    // Remove namespace
-    if (context && context->isNamespace() && filename.startsWith(context->name(), Qt::CaseInsensitive)) {
-        filename = filename.remove(0, context->name().size() + 1);
-    }
     QString sourceFile = documentationDataDirectory() + '/' + filename + ".xml";
     if (metaClass->enclosingClass())
         sourceFile.replace("::", "-");
@@ -84,7 +63,7 @@ void QtDocParser::fillDocumentation(AbstractMetaClass* metaClass)
 
     // Class/Namespace documentation
     QString type = metaClass->isNamespace() ? QLatin1String("namespace") : QLatin1String("class");
-    QString query = "/WebXML/document/" + type + "[@" + "name" + "=\"" + className + "\"]/description";
+    QString query = "/WebXML/document/" + type + "[@name=\"" + className + "\"]/description";
 
     DocModificationList signedModifs, classModifs;
     foreach (DocModification docModif, metaClass->typeEntry()->docModifications()) {
@@ -164,3 +143,25 @@ void QtDocParser::fillDocumentation(AbstractMetaClass* metaClass)
         meta_enum->setDocumentation(doc);
     }
 }
+
+Documentation QtDocParser::retrieveModuleDocumentation(const QString& name)
+{
+    // TODO: This method of acquiring the module name supposes that the target language uses
+    // dots as module separators in package names. Improve this.
+    QString moduleName = QString(name).remove(0, name.lastIndexOf('.') + 1);
+    QString sourceFile = documentationDataDirectory() + '/' + moduleName.toLower() + ".xml";
+
+    if (!QFile::exists(sourceFile)) {
+        ReportHandler::warning("Can't find qdoc3 file for module "
+                               + name + ", tried: "
+                                                 + sourceFile);
+        return Documentation();
+    }
+
+    QXmlQuery xquery;
+    xquery.setFocus(QUrl(sourceFile));
+
+    // Module documentation
+    QString query = "/WebXML/document/page[@name=\"" + moduleName + "\"]/description";
+    return Documentation(getDocumentation(xquery, query, DocModificationList()));
+}
