@@ -28,6 +28,8 @@
 #include <QFile>
 #include <QXmlInputSource>
 #include "reporthandler.h"
+// #include <tr1/tuple>
+#include <algorithm>
 
 TypeDatabase::TypeDatabase() : m_suppressWarnings(true), m_apiVersion(0)
 {
@@ -371,29 +373,32 @@ PrimitiveTypeEntry *TypeDatabase::findPrimitiveType(const QString& name) const
 
 ComplexTypeEntry* TypeDatabase::findComplexType(const QString& name) const
 {
-    TypeEntry* entry = findType(name);
-    if (entry && entry->isComplex())
-        return static_cast<ComplexTypeEntry*>(entry);
-    else
-        return 0;
+    QList<TypeEntry*> entries = findTypes(name);
+    foreach (TypeEntry* entry, entries) {
+        if (entry && entry->isComplex())
+            return static_cast<ComplexTypeEntry*>(entry);
+    }
+    return 0;
 }
 
 ObjectTypeEntry* TypeDatabase::findObjectType(const QString& name) const
 {
-    TypeEntry* entry = findType(name);
-    if (entry && entry->isObject())
-        return static_cast<ObjectTypeEntry*>(entry);
-    else
-        return 0;
+    QList<TypeEntry*> entries = findTypes(name);
+    foreach (TypeEntry* entry, entries) {
+        if (entry && entry->isObject())
+            return static_cast<ObjectTypeEntry*>(entry);
+    }
+    return 0;
 }
 
 NamespaceTypeEntry* TypeDatabase::findNamespaceType(const QString& name) const
 {
-    TypeEntry* entry = findType(name);
-    if (entry && entry->isNamespace())
-        return static_cast<NamespaceTypeEntry*>(entry);
-    else
-        return 0;
+    QList<TypeEntry*> entries = findTypes(name);
+    foreach (TypeEntry* entry, entries) {
+        if (entry && entry->isNamespace())
+            return static_cast<NamespaceTypeEntry*>(entry);
+    }
+    return 0;
 }
 
 bool TypeDatabase::supportedApiVersion(double version) const
@@ -412,3 +417,75 @@ void TypeDatabase::setDropTypeEntries(QStringList dropTypeEntries)
     m_dropTypeEntries.sort();
 }
 
+// Using std::pair to save some memory
+// the pair means (revision, typeIndex)
+// This global variable exists only because we can't break the ABI
+typedef QHash<const TypeEntry*, std::pair<int, int> > TypeRevisionMap;
+Q_GLOBAL_STATIC(TypeRevisionMap, typeEntryFields);
+static bool computeTypeIndexes = true;
+static int maxTypeIndex;
+
+int getTypeRevision(const TypeEntry* typeEntry)
+{
+    return typeEntryFields()->value(typeEntry).first;
+}
+
+void setTypeRevision(TypeEntry* typeEntry, int revision)
+{
+    (*typeEntryFields())[typeEntry].first = revision;
+    computeTypeIndexes = true;
+}
+
+static bool compareTypeEntriesByName(const TypeEntry* t1, const TypeEntry* t2)
+{
+    return t1->qualifiedCppName() < t2->qualifiedCppName();
+}
+
+int getTypeIndex(const TypeEntry* typeEntry)
+{
+    if (computeTypeIndexes) {
+        TypeDatabase* tdb = TypeDatabase::instance();
+        typedef QMap<int, QList<TypeEntry*> > GroupedTypeEntries;
+        GroupedTypeEntries groupedEntries;
+
+        // Group type entries by revision numbers
+        TypeEntryHash allEntries = tdb->allEntries();
+        foreach (QList<TypeEntry*> entryList, allEntries) {
+            foreach (TypeEntry* entry, entryList) {
+                if (entry->isPrimitive()
+                    || entry->isContainer()
+                    || entry->isFunction()
+                    || !entry->generateCode()
+                    || entry->isEnumValue()
+                    || entry->isVarargs()
+                    || entry->isTypeSystem()
+                    || entry->isVoid()
+                    || entry->isCustom())
+                    continue;
+                groupedEntries[getTypeRevision(entry)] << entry;
+            }
+        }
+
+        maxTypeIndex = 0;
+        GroupedTypeEntries::iterator it = groupedEntries.begin();
+        for (; it != groupedEntries.end(); ++it) {
+            // Remove duplicates
+            QList<TypeEntry*>::iterator newEnd = std::unique(it.value().begin(), it.value().end());
+            it.value().erase(newEnd, it.value().end());
+            // Sort the type entries by name
+            qSort(it.value().begin(), newEnd, compareTypeEntriesByName);
+
+            foreach (TypeEntry* entry, it.value()) {
+                (*typeEntryFields())[entry].second = maxTypeIndex++;
+            }
+        }
+    }
+
+    return typeEntryFields()->value(typeEntry).second;
+}
+
+int getMaxTypeIndex()
+{
+    return maxTypeIndex;
+}
+
