@@ -409,6 +409,7 @@ bool Handler::startElement(const QString &, const QString &n,
         QHash<QString, QString> attributes;
         attributes["name"] = QString();
         attributes["since"] = QString("0");
+        attributes["revision"] = QString("0");
 
         switch (element->type) {
         case StackElement::PrimitiveTypeEntry:
@@ -422,7 +423,8 @@ bool Handler::startElement(const QString &, const QString &n,
             attributes["type"] = QString();
             break;
         case StackElement::EnumTypeEntry:
-            attributes["flags"] = "no";
+            attributes["flags"] = QString();
+            attributes["flags-revision"] = QString();
             attributes["upper-bound"] = QString();
             attributes["lower-bound"] = QString();
             attributes["force-integer"] = "no";
@@ -526,6 +528,7 @@ bool Handler::startElement(const QString &, const QString &n,
             m_error = "no 'name' attribute specified";
             return false;
         }
+
         switch (element->type) {
         case StackElement::PrimitiveTypeEntry: {
             QString targetLangName = attributes["target-lang-name"];
@@ -591,10 +594,19 @@ bool Handler::startElement(const QString &, const QString &n,
             m_currentEnum->setExtensible(convertBoolean(attributes["extensible"], "extensible", false));
 
             // put in the flags parallel...
-            if (!attributes["flags"].isEmpty() && attributes["flags"].toLower() != "no") {
+            QString flagName = attributes["flags"];
+            if (!flagName.isEmpty()) {
                 FlagsTypeEntry *ftype = new FlagsTypeEntry("QFlags<" + name + ">", since);
                 ftype->setOriginator(m_currentEnum);
-                ftype->setOriginalName(attributes["flags"]);
+                // Try to get the guess the qualified flag name
+                if (!flagName.contains("::") && names.count() > 1) {
+                    QStringList cpy(names);
+                    cpy.removeLast();
+                    cpy.append(flagName);
+                    flagName = cpy.join("::");
+                }
+
+                ftype->setOriginalName(flagName);
                 ftype->setCodeGeneration(m_generate);
                 QString n = ftype->originalName();
 
@@ -610,6 +622,9 @@ bool Handler::startElement(const QString &, const QString &n,
 
                 m_database->addFlagsType(ftype);
                 m_database->addType(ftype);
+
+                QString revision = attributes["flags-revision"].isEmpty() ? attributes["revision"] : attributes["flags-revision"];
+                setTypeRevision(ftype, revision.toInt());
             }
         }
         break;
@@ -737,10 +752,12 @@ bool Handler::startElement(const QString &, const QString &n,
             Q_ASSERT(false);
         };
 
-        if (element->entry)
+        if (element->entry) {
             m_database->addType(element->entry);
-        else
+            setTypeRevision(element->entry, attributes["revision"].toInt());
+        } else {
             ReportHandler::warning(QString("Type: %1 was rejected by typesystem").arg(name));
+        }
 
     } else if (element->type == StackElement::InjectDocumentation) {
         // check the XML tag attributes
@@ -816,6 +833,7 @@ bool Handler::startElement(const QString &, const QString &n,
                         || element->type == StackElement::Rejection
                         || element->type == StackElement::LoadTypesystem
                         || element->type == StackElement::InjectCode
+                        || element->type == StackElement::ExtraIncludes
                         || element->type == StackElement::ConversionRule
                         || element->type == StackElement::AddFunction
                         || element->type == StackElement::Template;
@@ -991,11 +1009,8 @@ bool Handler::startElement(const QString &, const QString &n,
             }
             QString name = attributes["name"];
 
-            bool added = false;
-            if (!name.isEmpty() && m_database->supportedApiVersion(since)) {
-                added = true;
+            if (!name.isEmpty() && m_database->supportedApiVersion(since))
                 m_currentEnum->addEnumValueRejection(name);
-            }
 
         } break;
         case StackElement::ReplaceType: {
